/*
 * Copyright (C) 2007-2018 University of Stuttgart, IKR
 * 
 * This file is part of IKR SimLib, Java Edition, as available from
 * http://www.ikr.uni-stuttgart.de/IKRSimLib.
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License (LGPL) as published by the
 * Free Software Foundation, in version 2.1 as it comes in the file "COPYING" in
 * directory doc of IKR SimLib.
 * 
 * IKR SimLib is distributed in the hope that it will be useful, but without any
 * warranty of any kind. See the LGPL for details or contact the IKR if you need
 * additional information (http://www.ikr.uni-stuttgart.de).
 * 
 * Alternatively you may license IKR SimLib under a different (non-free)
 * license. Please contact the IKR for further details.
 */

package ikr.simlib.example.ackNackModel.components;

import ikr.simlib.entities.Entity;
import ikr.simlib.example.ackNackModel.messages.AckNackMessage;
import ikr.simlib.messages.Message;
import ikr.simlib.model.SimNode;
import ikr.simlib.ports.input.InputPort;
import ikr.simlib.ports.output.SynchronousOutputPort;

import java.util.ArrayDeque;
import java.util.Deque;

public class Sender extends Entity {

   private final SynchronousOutputPort output;
   private final InputPort input;
   private final InputPort feedbackInput;

   private final Deque<Message> queue;
   private Message messageInFlight;

   public Sender(SimNode ownNode) {
      super(ownNode);

      this.queue = new ArrayDeque<>();

      this.output = new SynchronousOutputPort(this);

      this.input = new InputPort(this) {
         @Override
         protected void handleMessageIndication(Message msg) {
            final Message m = getMessage();
            Sender.this.queue.add(m);
            trigger();
         }
      };

      this.feedbackInput = new InputPort(this) {
         @Override
         protected void handleMessageIndication(Message msg) {
            final AckNackMessage m = (AckNackMessage) getMessage();
            if (!m.isSuccessful()) {
               Sender.this.queue.push(Sender.this.messageInFlight);
            }
            Sender.this.messageInFlight = null;
            trigger();
         }
      };
   }

   private void trigger() {
      if (this.messageInFlight == null) {
         if (!this.queue.isEmpty()) {
            this.messageInFlight = this.queue.pop();
            this.output.sendMessage(this.messageInFlight);
         }
      }
   }

   public SynchronousOutputPort getOutput() {
      return this.output;
   }

   public InputPort getInput() {
      return this.input;
   }

   public InputPort getFeedbackInput() {
      return this.feedbackInput;
   }
}
